import hashlib
import requests
import os
import shutil

def image_path(root_path, img_file_name):
    """ Generate path for image file name within 'images' folder """
    return os.path.join(root_path, 'images', img_file_name)

def get_sha(image_path):
    """ Generate sha1 for a given file """
    sha1 = hashlib.sha1()
    f = open(image_path, 'rb')
    try:
        sha1.update(f.read())
    finally:
        f.close()
    return sha1.hexdigest() + '.jpg'

def photos_present_check(desktop, mobile, editorial=None, is_premium=False):
    """ Indicate whether all required image files are present """
    if not ((os.path.isfile(desktop))
            and (os.path.isfile(mobile)) 
            and (os.path.isfile(editorial) if is_premium else True)):
        return False
    return True

def prepare(
    root_path, 
    photos_present, 
    photo_location, 
    uuid=None, 
    is_premium=False, 
    desktop_photo_path=None, 
    mobile_photo_path=None, 
    editorial_photo_orig_path=None
):
    """ Verify presence of necessary photos; create Sha-named copies in
    app's 'images' folder 
    """
    for key in photos_present.keys():
        if uuid in key:
            if "desktop" in key:
                desktop_photo_path = os.path.join(photo_location, key)
            elif "mobile" in key:
                mobile_photo_path = os.path.join(photo_location, key)
            elif "editorial" in key:
                editorial_photo = key
                editorial_photo_orig_path = os.path.join(photo_location, key)
    if is_premium:
        if not photos_present_check(
            desktop_photo_path,
            mobile_photo_path, 
            editorial_photo_orig_path,
            is_premium
        ):
            return (None, None, None, None, None, None)
    else:
        if not photos_present_check(
            desktop_photo_path,
            mobile_photo_path
        ):
            return (None, None, None, None)
    sha_desktop_photo = get_sha(desktop_photo_path)
    sha_desktop_photo_path = image_path(root_path, sha_desktop_photo)
    shutil.copy2(desktop_photo_path, sha_desktop_photo_path)
    sha_mobile_photo = get_sha(mobile_photo_path)
    sha_mobile_photo_path = image_path(root_path, sha_mobile_photo)
    shutil.copy2(mobile_photo_path, sha_mobile_photo_path)
    if is_premium:
        editorial_photo_path = image_path(root_path, editorial_photo)
        shutil.copy2(editorial_photo_orig_path, editorial_photo_path)
        return (
            sha_desktop_photo, 
            sha_desktop_photo_path, 
            sha_mobile_photo, 
            sha_mobile_photo_path, 
            editorial_photo, 
            editorial_photo_path
        )
    return (
        sha_desktop_photo, 
        sha_desktop_photo_path, 
        sha_mobile_photo,
        sha_mobile_photo_path
    )

def initialize_photos_present(uuid, is_premium=False):
    """ Generate dictionary of booleans to verify presence of required
         image files. If is_premium is true, initialize 3 files; if not,
        initialize 2 files.
    """
    desktop_key = "%s_cover-desktop.jpg" % uuid
    mobile_key = "%s_cover-mobile.jpg" % uuid
    if is_premium: 
        editorial_key = "%s_editorial.jpg" % uuid
    return ({ 
        desktop_key: False, 
        mobile_key: False, 
        editorial_key: False 
    } if is_premium else {
        desktop_key: False, 
        mobile_key: False
    })

def refresh_photos_present(photos_present, path):
    """ Update dictionary based on presence of required image files. """
    for key in photos_present.keys():
        file_path = os.path.join(path, key)
        photos_present[key] = True if os.path.isfile(file_path) else False
    return photos_present