from datetime import datetime
import messages
from slackclient import SlackClient
import threading
import time

class Heartbeat:
    """ Thread that pings Slack to keep connection alive
    The run() method will be started and it will run in the background
    until the application exits.
    """

    def __init__(self, client, channel, interval=10, success_message=False):
        """ Constructor
        :type interval: int
        :param interval: Check interval, in seconds
        :type success_message: boolean
        :param success_message: Whether or not to notify on successful ping
        """
        self.client = client
        self.channel = channel
        self.interval = interval
        self.success_message = success_message

        thread = threading.Thread(target=self.run, args=())
        thread.daemon = True                            # Daemonize thread
        thread.start()                                  # Start the execution

    def run(self):
        """ Method that runs forever """
        while True:
            # When necessary, log success/failure of Slack client ping
            try:
                self.client.api_call("channels.info", channel = self.channel)
                if self.success_message: 
                    print(
                        "Successfully pinged Slack at %s." 
                        % str(datetime.now())
                    )
                    self.success_message = False
            except(Exception):
                print("Failure to ping Slack at %s." 
                    % str(datetime.now()))
                if not self.success_message: self.success_message = True
            time.sleep(self.interval)

def start(cred):
    """ Use Slack credentials to instantiate client """
    return SlackClient(cred)

def set_channel_topic(client, channel, app_name, running):
    """ Sets Slack channel topic """
    client.api_call(
        "channels.setTopic", 
        channel=channel, 
        topic=(
            messages.bot_running(app_name) 
            if running else (
                messages.bot_unavailable() 
                if running is None else messages.bot_available()
            )
        )
    )

def attachment(title, text, is_green=False):
    """ Returns object that styles public channel communications """
    color = "#36a64f" if is_green else "#a63636"
    return [
        {
            "fallback": "",
            "color": color,
            "title": title,
            "text": text,
            "mrkdwn_in": ["text"],
        }
    ]

def send_thread_message(
    client, contact_channel, contact_thread, text, reset = None
):
    """ Send a message to a particular message thread """
    client.api_call(
        "chat.postMessage", 
        username="qa-bot", 
        icon_emoji=":money_with_wings:", 
        channel=contact_channel, 
        thread_ts=contact_thread, 
        parse="full", 
        link_names="true", 
        text=text
    )
    if reset: return reset

def send_thread_message_for_reaction(
    client, contact_channel, contact_thread, text
):
    """ Send thread message that requires an emoji reaction """
    response = client.api_call(
        "chat.postMessage", 
        username="qa-bot", 
        icon_emoji=":money_with_wings:", 
        channel=contact_channel, 
        thread_ts=contact_thread, 
        parse="full", 
        link_names="true", 
        text=text
    )
    return response["ts"], None

def send_channel_help_message(
    client, channel, app_name, text, is_green=True
):
    """ Send the app's help message to the channel """
    client.api_call(
        "chat.postMessage", 
        username="qa-bot", 
        icon_emoji=":money_with_wings:", 
        attachments=attachment(app_name, text, is_green), 
        channel=channel, 
        parse="full", 
        link_names="true", 
        text=messages.help_intro())

def send_channel_message(
    client, channel, app_name, text, reset=None, is_end_of_process=False
):
    """ Send a message to a public channel """
    client.api_call(
        "chat.postMessage", 
        username="qa-bot", 
        icon_emoji=":money_with_wings:", 
        parse="full", 
        link_names="true", 
        attachments=attachment(app_name, text, is_end_of_process), 
        channel=channel
    )
    if reset: return reset

def send_channel_message_for_reaction(
    client, contact_channel, app_name, text
):
    """ Send channel message that requires an emoji reaction """
    response = client.api_call(
        "chat.postMessage",
        username="qa-bot", 
        icon_emoji=":money_with_wings:",
        channel=contact_channel, 
        parse="full", 
        link_names="true", 
        attachments=attachment(app_name, text)
    )
    return response["ts"], None

def send_first_dm_for_input(
    client, channel, user_id, app_name, instructions, intro
):
    """ Send initial private message requesting user input """
    direct_message = client.api_call("im.open", user=user_id)
    channel = direct_message["channel"]["id"]
    response = client.api_call(
        "chat.postMessage", 
        username="qa-bot", 
        icon_emoji=":money_with_wings:", 
        attachments=attachment(app_name, instructions),
        channel=channel, 
        parse="full", 
        link_names="true", 
        text=intro
    )
    return channel, response["ts"], None

def close_direct_message(client, channel):
    """ Close a DM channel """
    client.api_call("im.close", channel = channel)

def handle_message_submit(contact_thread, contact_user, slack_message):
    """ Pass back user input from Slack 
    if input is a message from the expected user 
    that has been sent on the expected thread
    """
    message_type = slack_message.get("type")
    message_user = slack_message.get("user")
    message_thread = slack_message.get("thread_ts")
    if (
        message_type == "message" 
        and message_user == contact_user 
        and message_thread == contact_thread):
        return slack_message.get("text").lower()
    return None

def parse_reaction(
    orig_thread, slack_message, valid_users, user_reaction=None, user=None
):
    """ Determine whether an approved user has reacted to a particular 
     message thread with an approved positive or negative emoji response
    """
    positives = ["+1", "wip", "wipit"]
    negatives = ["-1", "thumbsdown", "cgt"]
    message_type = slack_message.get("type")
    if message_type == "reaction_added":
        reaction = slack_message.get("reaction")
        item = slack_message.get("item")
        user = slack_message.get("user")
        if (
            item 
            and "ts" in item 
            and item["ts"] == orig_thread 
            and user in valid_users
        ):
            user_reaction = True if reaction in positives else (
                False if reaction in negatives else None
            )
        elif item and not "ts" in item: print item
    return user_reaction, user if user_reaction is not None else None

def basic_reaction_loop(
    client, channel, app_name, users, text=None, thread=None
):
    """ Wait for user to react to a prompt with an emoji """
    reaction_thread, reaction = (
        send_thread_message_for_reaction(client, channel, thread, text)
        if thread else 
        send_channel_message_for_reaction(client, channel, app_name, text)
    )
    while reaction is None:
        for slack_message in client.rtm_read():
            reaction, user = parse_reaction(
                reaction_thread, slack_message, users
            )
    return reaction

def retry_reaction_loop(
    client, 
    channel, 
    app_name, 
    users, 
    text=None, 
    thread=None, 
    approvals_required=1
):
    """ Wait for user to react, stopping current process 
    if user reacts with negative 
    """
    retry_thread, retry = (
        send_thread_message_for_reaction(
            client, 
            channel, 
            thread, 
            text
        ) if thread else send_channel_message_for_reaction(
            client, 
            channel, 
            app_name, 
            text
        )
    )
    for x in range(0, approvals_required):
        while retry is None:
            for message in client.rtm_read():
                retry, user = parse_reaction(
                    retry_thread, 
                    message, 
                    users
                )
        if not retry:
            (
                send_thread_message(
                    client=client, 
                    contact_channel=channel, 
                    contact_thread=thread, 
                    text=messages.quit_run()
                )
                if thread else send_channel_message(
                    client=client, 
                    channel=channel, 
                    app_name=app_name, 
                    text=messages.quit_run()
                )
            )
            return True
        users.remove(user)
        retry = None
    return False

def get_available_emoji(client, rihanna_mode=False):
    """ Return a list of all custom emoji, 
    filtering out aliases (which are not accessible as reactions).
    If rihanna_mode is enabled, only return Rihanna-based emoji.
    Otherwise return all non-Rihanna-based custom emoji, because
    the Rihanna emoji are so similarly named that makes
    the nuclear codes overly difficult to read.
    """
    emoji_list = []
    response = client.api_call("emoji.list")
    if response["emoji"]:
        emoji_json = response["emoji"]
        for key, val in emoji_json.items():
            if "alias:" not in val:
                if rihanna_mode: 
                    if "rih" in key: emoji_list.append(key)
                else:
                    if not "rih" in key: emoji_list.append(key)
    return emoji_list

def get_nuclear_reaction(orig_thread, slack_message, origin_user):
    """ Receive user reaction to a specific message thread and return it """
    message_type = slack_message.get("type")
    if message_type == "reaction_added":
        reaction = slack_message.get("reaction")
        item = slack_message.get("item")
        user = slack_message.get("user")
        if (
            item 
            and "ts" in item 
            and item["ts"] == orig_thread 
            and user == origin_user
        ):
            return reaction
    return None

def nuclear_reaction_loop(
    client, 
    channel, 
    user, 
    inputs_required, 
    app_name=None, 
    thread=None, 
    text=None
):
    """ Wait for user to submit sequence of emoji that form
    the approval code in nuclear mode
    """
    retry_thread, retry = (
        send_thread_message_for_reaction(
            client=client, 
            contact_channel=channel, 
            contact_thread=thread, 
            text=text
        ) if thread else send_channel_message_for_reaction(
            client=client, 
            contact_channel=channel,
            app_name=app_name,
            text=text
        )
    )
    emoji_code = []
    for x in range(0, inputs_required):
        reaction = None
        while reaction is None:
            for slack_message in client.rtm_read():
                reaction = get_nuclear_reaction(
                    orig_thread=retry_thread, 
                    slack_message=slack_message, 
                    origin_user=user
                )
        emoji_code.append(reaction)
    return emoji_code